ball = {}

ball_radius = screen.width / 64

local ball_fadeSpeed = 512
local ball_scaleSpeed = 10
local ball_color = {0, 128, 255, 0}
local ball_duration = 6

ball_type = {"basic", "scale_up", "scale_down", "double_score", "scale_half", "scale_double"}


local bm = {__index = ball}

function ball.spawn(x, y, type)
	local b = {
			x = x,
			y = y,
			radius = ball_radius,
			type = type,
			color = {ball_color[1], ball_color[2], ball_color[3], ball_color[4]},
			initialized = false,
			collected = false,
			remove = false,
			tick = 0
		}
	
	return setmetatable(b, bm)
end

function ball:update(dt)
	--Color Update
	self.color[1], self.color[2], self.color[3] = game.background.color[1], game.background.color[2], game.background.color[3] + 26
	
	if not self.initialized then
		self.color[4] = self.color[4] + (ball_fadeSpeed * 2) * dt
		if self.color[4] > 200 then
			self.color[4] = 200
			self.initialized = true
		end
	else		
		--PLAYER COL
		if self:collide(player.x, player.y, player.radius) then
			self:pickup()
		end
		
		if not self.collected then
			self.tick = self.tick + dt
			if self.tick > ball_duration then
				self.collected = true
			end
		else
			self.color[4] = self.color[4] - ball_fadeSpeed * dt
			self.radius = self.radius + ball_scaleSpeed * dt
			if self.color[4] < 0 then
				self.color[4] = 0
				self.remove = true
			end
		end
	end
end

function ball:draw()
	love.graphics.setColor(hsl(self.color))
	--love.graphics.setBlendMode("alpha")
	local seg = self.radius
	if self.type == "scale_up" then seg = 4  elseif self.type == "scale_down" then seg = 5 elseif self.type == "double_score" then seg = 6 elseif self.type == "scale_half" then seg = 7
	elseif self.type == "scale_double" then seg = 8 end
	love.graphics.circle("fill", self.x, self.y, self.radius, seg)
	
	love.graphics.setColor(hsl(self.color[1], self.color[2], self.color[3] + 16, self.color[4]))		
	love.graphics.circle("line", self.x, self.y, self.radius, seg)
	
	--HELP
	love.graphics.setColor(0, 0, 0, self.color[4] / 2)
	love.graphics.setFont(font.small)
	local s = "b"
	--{"basic", "scale_up", "scale_down", "double_score", "scale_half", "scale_double"}
	if self.type == "scale_up" then
		s = "su"
	elseif self.type == "scale_down" then
		s = "sd"
	elseif self.type == "double_score" then
		s = "ds"
	elseif self.type == "scale_half" then
		s = "sh"
	elseif self.type == "scale_double" then
		s = "scd"
	end
	--love.graphics.print(s, self.x - (love.graphics.getFont():getWidth(s) / 2), self.y - (love.graphics.getFont():getHeight() / 2))
	--love.graphics.setBlendMode("alpha")
end

function ball:pickup()
	if not self.collected then 
		gui:addCircle("line", self.x, self.y, self.radius, 10, ball_fadeSpeed / 2) 
		if self.type == "basic" then
			game.score = game.score + 1
		elseif self.type == "scale_up" then
			player.nradius = player.radius + (player.radius / 8)
			gui:addNote("player size increased", font.small)
		elseif self.type == "scale_down" then
			player.nradius = player.radius - (player.radius / 8)
			gui:addNote("player size decreased", font.small)
		elseif self.type == "double_score" then
			game.score = game.score * 2
			gui:addNote("score doubled!", font.medium)
		elseif self.type == "scale_half" then
			player.nradius = player.radius / 2
			gui:addNote("player size halfed", font.medium)
		elseif self.type == "scale_double" then
			player.nradius = player.radius * 2
			gui:addNote("player size doubled", font.medium)
		end
		game.player.scale_tick = game.player.scale_tick + 1
		if game.player.scale_tick > game.player.scale_interval then
			player.nradius = player.radius + (player.radius / 8)
			gui:addNote("player size increased", font.small)
			gui:addCircle("line", player.x, player.y, player.radius, 50)
			game.player.scale_tick = 0
		end
	end
	self.collected = true
	
end

function ball:collide(x, y, radius)
	local dist = getDistance(x, y, self.x, self.y)
	return dist < self.radius + radius
end















