game = {
		started = false,
		paused = false,
		time = 0,
		tick = 0,
		score = 0,
		canvas = {
			love.graphics.newCanvas(screen.width, screen.height),
			love.graphics.newCanvas(screen.width, screen.height)
		},
		slow = false,
		time_scale = 1,
		min_time_scale = 0.001,
		
		start_countdown = {
				enabled = false,
				time = 3,
				_time = 0,
				tick = 0
			},
		background = {
				color = {0, 126, 100, 255},
				min_saturation = 10,
				max_saturation = 126,
				fade_speed = 300,
				speed = 8
			},
			
		balls = {
			spawned = {},
			tick = 0,
			spawn_rate = 10,
			start_balls = 16
		},
		hud = {
			radius = screen.width / 20,
			x = screen.width / 2,
			y = (screen.width / 20) + ((screen.width / 20) / 8),
			color = {
					bg = {255, 255, 255, 50}, 
					outline = {255, 255, 255, 100}, 
					text = {0, 0, 0, 255},
				}
		},

		legend = {
			show = true,
			x = (screen.width / 2) - (200 / 2),
			y = (screen.height / 2) - (450 / 2),
			width = 200,
			height = 450,
			alpha = 0,
			fade_speed = 10,
			items = {}
		},
		player = {
			scale_interval = 150,
			scale_tick = 0
		},
		shader = {
			split = {
					enabled = false,
					amount = 0,
					angle = 0
				},
			blur = {
					enabled = true,
					intensity = 0,
					max_intensity = 0.4,
					fade_speed = 4
				}
			}
		
	}
	

function game:initialize()
	player:initialize()
	self.started = true
	if self.start_countdown.enabled then self.started = false end
	self.paused = false
	self.start_countdown.tick = 0
	self.start_countdown._time = self.start_countdown.time
	self.background.color[1] = love.math.random(255)
	--self.balls.color[1], self.balls.color[2], self.balls.color[3] = self.background.color[1], self.background.color[2], self.background.color[3] + 26
	
	love.graphics.setBackgroundColor(hsl(self.background.color))
	
	self.time = 0
	self.tick = 0
	self.score = 0
	
	for i=1, self.balls.start_balls do
		self.balls.spawned[#self.balls.spawned + 1] = ball.spawn(love.math.random(ball_radius, screen.width - ball_radius), love.math.random(ball_radius, screen.height - ball_radius), ball_type[1])
	end
	
	--LEGEND {"basic", "scale_up", "scale_down", "double_score", "scale_half", "scale_double"}
	for i,v in ipairs(ball_type) do
		local c = i + 1
		local seg = ball_radius
		if v == "scale_up" then
			seg = 4
		elseif v == "scale_down" then
			seg = 5
		elseif v == "double_score" then
			seg = 6
		elseif v == "scale_half" then
			seg = 7
		elseif v == "scale_double" then
			seg = 8
		end
		self.legend.items[#self.legend.items + 1] = {
				x = self.legend.x + 12,
				y = (self.legend.y / 2) * c,
				name = v,
				segments = seg
			}
	end
end


local set = false

function game:update(dt)
	--Start Countdown
	if not self.started then
		self.start_countdown.tick = self.start_countdown.tick + dt
		if self.start_countdown.tick > 1 then
			self.start_countdown._time = self.start_countdown._time - 1
			--(draw_mode, x, y, radius, scale_speed, fade_speed)
			gui:addCircle("fill", screen.width / 2, screen.height / 2, player.radius, 100, 200)
			if self.start_countdown._time <= 0 then
				self.started = true
				self.start_countdown._time = 0
			end
			
			self.start_countdown.tick = 0
		end
	end


	if self.started then
	
		player:update(dt)
			
		--Ball Spawning
		self.balls.tick = self.balls.tick + dt
		if self.balls.tick > 1 / self.balls.spawn_rate then
			local r = love.math.random(1000)
			local t = 1
			if r < 30 then
				r = love.math.random(130)
				if r < 100 and r > 50 then
					t = 2
				elseif r < 50 then
				 t = 3
				elseif r > 100 and r < 110 then
					t = 4
				elseif r > 110 and r < 120 then
					t = 5
				elseif r > 120 then
					t = 6
				end
			end
			self.balls.spawned[#self.balls.spawned + 1] = ball.spawn(love.math.random(ball_radius, screen.width - ball_radius), love.math.random(ball_radius, screen.height - ball_radius), ball_type[t])
			print(ball_type[t])
			self.balls.tick = 0
		end
		
		--Balls
		for i,v in ipairs(self.balls.spawned) do
			v:update(dt)
			if v.remove then
				table.remove(self.balls.spawned, i)
			end
		end
		
		--GAME LOGIC
		self.tick = self.tick + dt
		if self.tick > 1 then
			self.time = self.time + 1
			gui:addCircle("line", self.hud.x, self.hud.y, self.hud.radius, 16)
			self.tick = 0
		end
	
	end
	
	--SHADER UPDATES
	self.shader.split.amount = 2 + (player.speed / 3000)
	
	shader.split:send("chroma", {self.shader.split.amount, self.shader.split.angle})
	shader.split:send("imageSize", {screen.width, screen.height})
	
	--blur
	shader.blur_vertical:send("intensity", self.shader.blur.intensity)
	shader.blur_vertical:send("rt_h", screen.height / 4)
	
	shader.blur_horizontal:send("intensity", self.shader.blur.intensity)
	shader.blur_horizontal:send("rt_w", screen.width / 8)
	
	self.slow = false
	self.legend.show = false
	if self.paused or not self.started then self.slow = true end
	if love.keyboard.isDown("tab") and self.started then 
		self.slow = true
		self.legend.show = true
	end
		
	--SLOW MOTION
	if self.slow then
		if self.started and not self.paused then
			self.time_scale = self.min_time_scale
		elseif self.paused then
			self.time_scale = 0
		end
		--BG COLOR
		game.background.color[2] = game.background.color[2] - game.background.fade_speed * (self.min_time_scale * 300000) * dt
		if game.background.color[2] < game.background.min_saturation then
			game.background.color[2] = game.background.min_saturation
		end
		--shader
		game.shader.blur.enabled = true
		game.shader.split.enabled = true
		game.shader.blur.intensity = game.shader.blur.intensity + game.shader.blur.fade_speed * (self.min_time_scale * 300000) * dt
		if game.shader.blur.intensity > game.shader.blur.max_intensity then
			game.shader.blur.intensity = game.shader.blur.max_intensity
		end
		
		--LEGEND
		if self.legend.show then
			self.legend.alpha = self.legend.alpha +  self.legend.fade_speed * (self.min_time_scale * 300000) * dt
			if self.legend.alpha > 1 then 
				self.legend.alpha = 1 
			end
			set = false
		end
	else --NORMAL
		self.time_scale = 1
		if not set then
			love.mouse.setPosition(player.x, player.y)
			set = true
		end
		game.background.color[2] = game.background.color[2] + game.background.fade_speed * dt
		if game.background.color[2] > game.background.max_saturation then
			game.background.color[2] = game.background.max_saturation
		end
		--shader
		game.shader.blur.intensity = game.shader.blur.intensity - game.shader.blur.fade_speed * dt
		if game.shader.blur.intensity < 0 then
			game.shader.blur.intensity = 0
			game.shader.blur.enabled = false
			game.shader.split.enabled = false
		end	
		--LEGEND
		self.legend.alpha = self.legend.alpha - self.legend.fade_speed * dt
		if self.legend.alpha < 0 then 
			self.legend.alpha = 0 
		end
	end
	
	--BG COLOR
	self.background.color[1] = self.background.color[1] + self.background.speed * dt
	if self.background.color[1] > 255 then
		self.background.color[1] = 0
	end
	love.graphics.setBackgroundColor(hsl(self.background.color))
	
	draw_game()
end

function draw_game()
	game.canvas[1]:clear()
	love.graphics.setCanvas(game.canvas[1])
	
	love.graphics.setColor(hsl(game.background.color))
	love.graphics.rectangle("fill", 0, 0, screen.width, screen.height)
	
	if vignette.enabled then
		love.graphics.setColor(255, 255, 255, vignette.alpha)
		love.graphics.draw(image.vignette, 0, 0, 0, screen.width / image.vignette:getWidth(), screen.height / image.vignette:getHeight())
	end
	
	for i,v in ipairs(game.balls.spawned) do
		v:draw()
	end
	player:draw()
	
	gui:draw()
	
	if not game.paused then
		draw_hud()
	end
	
	--VERTICAL BLUR PASS
	game.canvas[2]:clear()
	love.graphics.setCanvas(game.canvas[2])
	love.graphics.setColor(255, 255, 255, 255)
	love.graphics.setShader(shader.blur_vertical)
	love.graphics.draw(game.canvas[1])
	if game.shader.split.enabled then love.graphics.setShader(shader.split) end
	love.graphics.draw(game.canvas[2])
	
	love.graphics.setShader()
	
	love.graphics.setCanvas()
end

function draw_hud()
		--HUD
	love.graphics.setColor(game.hud.color.bg)
	love.graphics.circle("fill", game.hud.x, game.hud.y, game.hud.radius)

	love.graphics.setColor(game.hud.color.outline)
	love.graphics.circle("line", game.hud.x, game.hud.y, game.hud.radius)
	
	love.graphics.setFont(font.large)
	love.graphics.setColor(game.hud.color.text)
	love.graphics.print(game.time, game.hud.x - (love.graphics.getFont():getWidth(game.time) / 2), game.hud.y - love.graphics.getFont():getHeight() / 2)
	love.graphics.setFont(font.medium)
	love.graphics.print(game.score, game.hud.x - (love.graphics.getFont():getWidth(game.score) / 2), game.hud.y + love.graphics.getFont():getHeight() / 2)
	
	
end


function game:draw()
	love.graphics.setColor(255, 255, 255, 255)
	love.graphics.setBlendMode("alpha")
	
	if self.shader.blur.enabled then love.graphics.setShader(shader.blur_horizontal) end
	love.graphics.draw(game.canvas[2])
	love.graphics.setShader()
	
	if self.paused then
		draw_hud()
	end


	--LEGEND
	love.graphics.setColor(255, 255, 255, 50 * self.legend.alpha)
	love.graphics.rectangle("fill", self.legend.x, self.legend.y, self.legend.width, self.legend.height)
	love.graphics.rectangle("line", self.legend.x, self.legend.y, self.legend.width, self.legend.height)
	
	love.graphics.setFont(font.medium)
	for i,v in ipairs(self.legend.items) do
		love.graphics.setColor(0, 0, 0, 200 * self.legend.alpha)
		love.graphics.print(v.name, v.x + ball_radius * 2 + 12, v.y + ball_radius)
		love.graphics.circle("fill", v.x + ball_radius, v.y + (ball_radius * 2), ball_radius, v.segments)
	end

	--COUNTDOWN
	if not self.started then
		love.graphics.setFont(font.large)
		local txt = self.start_countdown._time
		love.graphics.setColor(0, 0, 0, 255)
		love.graphics.print(txt, (screen.width / 2) - (love.graphics.getFont():getWidth(txt) / 2), (screen.height / 2) - (love.graphics.getFont():getHeight() / 2))
	end
end

function game:keypressed(key)
	if key == "p" then
		if self.started then
			self.paused = not self.paused
			if not self.paused then
				love.mouse.setPosition(player.x, player.y)
			else
				gui:addNote("Paused", font.medium)
				--gui:addNote(text, font, duration, x, y)
			end
		end
	end
end

function game:mousepressed(x, y, key)

end