gui = {
		circle = {
				spawned = {},
				scale_speed = 32,
				fade_speed = 255
			},
		note = {
				spawned = {},
				fade_speed = 255,
				width = screen.width / 8,
				height = screen.height / 10,
				x = 12,
				y = screen.height - (screen.height / 10) + 16,
				duration = 2,
				color = {
						bg = {255, 255, 255, 50},
						outline = {255, 255, 255, 100},
						text = {0, 0, 0, 255}
					}
			}
	}


function gui:addCircle(draw_mode, x, y, radius, scale_speed, fade_speed)
	self.circle.spawned[#self.circle.spawned + 1] = {
			x = x,
			y = y,
			radius = radius,
			scale_speed = scale_speed or self.circle.scale_speed,
			fade_speed = fade_speed or self.circle.fade_speed,
			color = {mouse.color[1], mouse.color[2], mouse.color[3], mouse.color[4]},
			draw_mode = draw_mode or "line"
		}
end

function gui:addNote(text, font, duration, x, y)
	local h = font:getHeight() + 12
	local w = font:getWidth(text) + 24

	self.note.spawned[#self.note.spawned + 1] = {
			text = text,
			font = font,
			x = x or self.note.x,
			y = y or (self.note.y - (h + 12) * #self.note.spawned),
			nx = x or self.note.x,
			ny = y or (self.note.y - (h + 12) * #self.note.spawned),
			width = w,
			height = h,
			initialized = false,
			duration = duration or self.note.duration,
			expired = false,
			alpha = 0,-- 0-1
			tick = 0
			
		}
end

function gui:initialize()

end

function gui:update(dt)
	if game.paused then dt = love.timer.getDelta() end
	--Circles
	for i,v in ipairs(self.circle.spawned) do
		v.radius = v.radius + v.scale_speed * dt
		v.color[4] = v.color[4] - v.fade_speed * dt
		if v.color[4] < 0 then
			v.color[4] = 0
			table.remove(self.circle.spawned, i)
		end
	end
	
	--Notifications
	for i,v in ipairs(self.note.spawned) do
		--init
		if not v.initialized then
			v.alpha = v.alpha + (self.note.fade_speed / 100) * dt
			if v.alpha > 1 then
				v.alpha = 1
				v.initialized = true
			end
		else
			--duration
			if not v.expired then
				v.tick = v.tick + dt
				if v.tick > v.duration then
					v.expired = true
				end
			else
				v.alpha = v.alpha - (self.note.fade_speed / 100) * dt
				if v.alpha < 0 then
					v.alpha = 0
					table.remove(self.note.spawned, i)
					gui:fixNotes()
				end
			end
		end
		--POSITION UPDATE
		if v.y < v.ny then
			v.y = v.y + (getDistance(v.y, v.x, v.nx, v.ny) / 3) * dt
			if v.y > v.ny then
				v.y = v.ny
			end
		elseif v.y > v.ny then
			v.y = v.y - (getDistance(v.y, v.x, v.nx, v.ny) / 3) * dt
			if v.y < v.ny then
				v.y = v.ny
			end
		end
	end
	
end

function gui:draw()
	--CIRCLES
	for i,v in ipairs(self.circle.spawned) do
		love.graphics.setColor(hsl(v.color))
		love.graphics.circle(v.draw_mode, v.x, v.y, v.radius)
	end
	
	--Notifications
	for i,v in ipairs(self.note.spawned) do
		love.graphics.setBlendMode("alpha")
		--bg
		love.graphics.setColor(self.note.color.bg[1], self.note.color.bg[2], self.note.color.bg[3], self.note.color.bg[4] * v.alpha)
		love.graphics.rectangle("fill", v.x, v.y, v.width, v.height)
		--outline
		love.graphics.setColor(self.note.color.outline[1], self.note.color.outline[2], self.note.color.outline[3], self.note.color.outline[4] * v.alpha)
		love.graphics.rectangle("line", v.x, v.y, v.width, v.height)
		
		love.graphics.setColor(self.note.color.text[1], self.note.color.text[2], self.note.color.text[3], self.note.color.text[4] * v.alpha)
		love.graphics.setFont(v.font)
		love.graphics.print(v.text, v.x + (v.width / 2) - (v.font:getWidth(v.text) / 2), v.y + (v.height / 2) - (v.font:getHeight() / 2))
		
	end
end

function gui:fixNotes()
	for i,v in ipairs(self.note.spawned) do
		--v.ny = self.note.y - (self.note.height + 12) * #self.note.spawned
		v.ny = v.y + v.height + 12
	end
end









