local fullscreen = false

if fullscreen then
	love.window.setFullscreen(true, "desktop")
end

screen = {
		width = love.graphics.getWidth(),
		height = love.graphics.getHeight()
	}
	
isShader = true

--Loading Files
require "shader"
require "mouse"
require "gui"
require "player"
require "ball"
require "game"
require "menu"	

--font
font = {
		small = love.graphics.newFont("data/font/font.ttf", 24),
		medium = love.graphics.newFont("data/font/font.ttf", 32),
		large = love.graphics.newFont("data/font/font.ttf", 52)
	}
for i,v in ipairs(font) do
	v:setLineHeight(0.1)
end

--img
image = {
		vignette = love.graphics.newImage("data/img/vignette.png")
	}
	
--GRAPHICS SETTINGS
vignette = {
		enabled = true,
		alpha = 255
	}

	
local state = {
		game = "game",
		menu = "menu"
	}
	
local game_state = state.game
	
function love.load()
	game:initialize()
	love.math.setRandomSeed(os.time())
end


function love.update(dt)
	--TIME SCALE
	dt = dt * game.time_scale
	
	mouse:update(dt)
	if game_state == state.game then
		game:update(dt)
	elseif game_state == state.menu then
		menu:update(dt)
	end
	
	--GUI
	gui:update(dt)
end

function love.draw()
	--vignette
	if vignette.enabled then
		love.graphics.setColor(255, 255, 255, vignette.alpha)
		love.graphics.draw(image.vignette, 0, 0, 0, screen.width / image.vignette:getWidth(), screen.height / image.vignette:getHeight())
	end

	if game_state == state.game then
		game:draw()
	elseif game_state == state.menu then
		menu:draw()
	end
	
	mouse:draw()
	
	love.graphics.setColor(255, 255, 255, 255)
	love.graphics.setFont(font.medium)
	--love.graphics.print("Balls: "..#game.balls.spawned.."\nPlayer Radius: "..math.floor(player.radius).."\nfps: "..love.timer.getFPS().."\nlegend: "..tostring(game.legend.show), 12, 12)
end

function love.keypressed(key)
	if key == "escape" then love.event.push("quit") end
	
	if key == " " then
		--gui:addNote("mblur: "..tostring(game.mblur.enabled), font.small)
		--gui:addNote(text, font, duration, x, y)
	end
	
	if game_state == state.game then
		game:keypressed(key)
	elseif game_state == state.menu then
		menu:keypressed(key)
	end
end

function love.mousepressed(x, y, key)
	if game_state == state.game then
		game:mousepressed(x, y, key)
	elseif game_state == state.menu then
		menu:mousepressed(x, y, key)
	end
	mouse:pressed(x, y, k)
	
	if key == "wu" then
		game.shader.blur.intensity = game.shader.blur.intensity + 0.1
	elseif key == "wd" then
		game.shader.blur.intensity = game.shader.blur.intensity - 0.1
	end
end

function getDistance(x1, y1, x2, y2)
	local dx = x1-x2
	local dy = y1-y2
	return math.sqrt((dx * dx + dy * dy))
end

function getAngle(x1, y1, x2, y2) 
	return math.atan2(x2-x1, y2-y1) 
end

function hsl(h, s, l, a)
	if type(h) == "table" then
		h,s,l,a = h[1], h[2], h[3], h[4] or 255
	end
    if s<=0 then return l,l,l,a end
    h, s, l = h/256*6, s/255, l/255
    local c = (1-math.abs(2*l-1))*s
    local x = (1-math.abs(h%2-1))*c
    local m,r,g,b = (l-.5*c), 0,0,0
    if h < 1     then r,g,b = c,x,0
    elseif h < 2 then r,g,b = x,c,0
    elseif h < 3 then r,g,b = 0,c,x
    elseif h < 4 then r,g,b = 0,x,c
    elseif h < 5 then r,g,b = x,0,c
    else              r,g,b = c,0,x
    end return (r+m)*255,(g+m)*255,(b+m)*255,a
end
